#!/usr/bin/env python

# Author:
#   Dominique Blaze <contact@d0m.tech>

import os
from import_handler import oscap, result2str, get_path


'''
Import and browse benchmark results

'''

benchmark = oscap.xccdf.benchmark_import(get_path("samples/xccdf_sample_results.xml"))
if benchmark.instance is None:
    raise Exception("Cannot import the benchmark {0}"
                    .format(get_path("samples/xccdf_sample_results.xml")))
else:
    print("Benchmark id: ", benchmark.get_id())
    results = benchmark.get_results()
    test_result = results.pop()
    print(test_result.get_id())

    for rs in test_result.get_rule_results():
        idref, result = rs.get_idref(), rs.get_result()
        print(idref, result2str(result))

        if (idref == "R-SHOULD_PASS" and
                result != oscap.xccdf.XCCDF_RESULT_PASS):

            raise Exception("Rule result for {0} should be PASS but is currently {1}."
                            .format(idref,
                                    result2str(result)))

        elif (idref == "R-SHOULD_FAIL" and
              result != oscap.xccdf.XCCDF_RESULT_FAIL):

            raise Exception("Rule result for {0} should be FAIL but is currently {1}."
                            .format(idref,
                                    result2str(result)))


# Now ensure that benchmark_import return None if the file doesn't exists

benchmark = oscap.xccdf.benchmark_import("../../oval_details/file_doesnt_exists.xml")
if benchmark.instance is not None:
    raise Exception("benchmark_import should return None if it can't open the file")
else:
    print("benchmark_import on a non existing file returns None.")



#!/usr/bin/env python

# Author:
#   Dominique Blaze <contact@d0m.tech>
#
# This script aims to verify the evaluation of an xccdf_session
#

import os
from import_handler import oscap, result2str, get_path


print("opening ./samples/xccdf_sample.xml ...")
sess = oscap.xccdf.session_new(get_path("samples/xccdf_sample.xml"))

res = sess.load()

if not res == 0:
    raise ValueError("Return value of xccdf_session_load isn't 0 but {0} !".format(res))

'''
rules to test whith desired return value
'''

rules = {"R-SHOULD_PASS": oscap.common.XCCDF_RESULT_PASS,
         "R-SHOULD_FAIL": oscap.common.XCCDF_RESULT_FAIL}

res = sess.evaluate()

if not res == 0:
    raise ValueError("Return value of xccdf_session_evaluate isn't 0 but {0} !".format(res))

if sess.get_base_score() != 50:
    raise ValueError("base score should be 50 but is {0} ", sess.get_base_score())

for rs in sess.get_xccdf_policy().get_results():
    for rr in rs.get_rule_results():
        print("Rule {0} evaluated as {1}"
              .format(rr.get_idref(), result2str(rr.get_result())))

        if rules[rr.get_idref()] != rr.get_result():
            raise ValueError("Result value of {0} should be {1} but is {2}"
                             .format(rr.get_idref(), rules[rr.get_idref()], rr.get_result()))

sess.free()



# Copyright (C) 2015 Brent Baude <bbaude@redhat.com>
# Copyright (C) 2019 Dominique Blaze <contact@d0m.tech>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301 USA

import subprocess
import platform
import os
import collections


class OscapError(Exception):
    ''' oscap Error'''
    pass

OscapResult = collections.namedtuple("OscapResult", ("returncode", "stdout", "stderr"))


def oscap_chroot(chroot_path, oscap_binary, oscap_args, target_name, local_env=[]):
        '''
        Wrapper running oscap_chroot on an OscapDockerScan OscapAtomicScan object
        '''
        os.environ["OSCAP_PROBE_ROOT"] = os.path.join(chroot_path)
        os.environ["OSCAP_EVALUATION_TARGET"] = target_name
        os.environ["OSCAP_CONTAINER_VARS"] = '\n'.join(local_env)

        cmd = [oscap_binary] + [x for x in oscap_args]

        oscap_process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE)
        oscap_stdout, oscap_stderr = oscap_process.communicate()
        return OscapResult(oscap_process.returncode,
                           oscap_stdout.decode("utf-8"),
                           oscap_stderr.decode("utf-8"))

# TODO replace by _get_cpe (in order to indentify any containerized system)


def get_dist(mountpoint, oscap_binary, local_env):
    CPE_RHEL = 'oval:org.open-scap.cpe.rhel:def:'
    DISTS = ["8", "7", "6", "5"]

    '''
    Test the chroot and determine what RHEL dist it is; returns
    an integer representing the dist
    '''

    cpe_dict = '/usr/share/openscap/cpe/openscap-cpe-oval.xml'
    if not os.path.exists(cpe_dict):
        # sometime it's installed into /usr/local/share instead of /usr/local
        cpe_dict = '/usr/local/share/openscap/cpe/openscap-cpe-oval.xml'
        if not os.path.exists(cpe_dict):
            raise OscapError()

    for dist in DISTS:
        result = oscap_chroot(
            mountpoint, oscap_binary,
            ("oval", "eval", "--id", CPE_RHEL + dist, cpe_dict,
             mountpoint, "2>&1", ">", "/dev/null"),
            '*',
            local_env
        )

        if "{0}{1}: true".format(CPE_RHEL, dist) in result.stdout:
            print("This system seems based on RHEL{0}.".format(dist))
            return dist



#!/usr/bin/env python

# Author:
#   Dominique Blaze <contact@d0m.tech>
#

'''
Basic OVAL evaluation:

Import an oval file with oscap.oval.definition_model_import_source
and run evalutation of some sample checks using a callback

Tested functions:
    - oscap.oval.definition_model_import_source
    - oscap.common.source_new_from_file
    - TODO oscap.oval.import_model
    - oscap.oval.agent_new_session
    - oscap.oval.agent_eval_system + associated callback
    - oval_result_definition.get_id()
    - oval_result_definition.get_result()
    - oval_result_definition.get_criteria()

Tested in oval_helpers.browse_criteria:
    - oval_result_criteria.get_type() + some of associated constants OVAL_NODETYPE_*
    - oval_result_criteria.get_subnodes()
    - oval_result_criteria.get_test()

    - oval_result_test.get_test()
    - oval_result_test.get_result()

    - oval_test.get_subtype() + some of associated constants OVAL_SUBYTPE_*
    - oval_test.get_family() + some of associated constants
'''

import os
import time
from import_handler import oscap, result2str, get_path
from oval_helpers import browse_criteria


'''
Intermediate functions
'''


# if you return something in callback else than 0, the current session stops
def oval_sample_callback(ovdef, usr):

    try:
        # .eval or .get_result return the same thing
        usr['results'].append(ovdef.get_id() + " => " + result2str(ovdef.get_result()))

        # retrieve the tests tree and replace test_results by [subtype => result]
        tests_tree = browse_criteria(ovdef.get_criteria(), 1)

        '''
        Expected tree for foo_pass: [[(7006, 1)]]
        = [[(OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, XCCDF_RESULT_PASS)]]
        or [[(OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, XCCDF_RESULT_PASS)]] for foo_fail
        '''

        expected_trees = {"oval:foo_pass:def:1": [[
            (oscap.oval.OVAL_FAMILY_INDEPENDENT,
             oscap.oval.OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54,
             oscap.oval.XCCDF_RESULT_PASS)]],

            "oval:foo_fail:def:1": [[
                (oscap.oval.OVAL_FAMILY_INDEPENDENT,
                 oscap.oval.OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54,
                 oscap.oval.XCCDF_RESULT_FAIL)]]
        }

        expected_tree = expected_trees.get(ovdef.get_id())

        if expected_tree is None:
            raise ValueError("Unexpected oval def : {0}. No expected tests tree defined for it"
                             .format(ovdef.get_id()))

        elif expected_tree == tests_tree:
            usr['results'].append("Tests tree of {0} is like expected : {1}".format(ovdef.get_id(),
                                                                                    tests_tree))

        else:
            raise ValueError("Tests tree of {0} doesn't match with the expected tree.\n"
                             "Extracted tree: {1}\nExpected tree: {2}"
                             .format(ovdef.get_id(), tests_tree, expected_tree))

    except Exception as e:
        usr['results'].append(e)

    return 0


# eval oval defs oval:[foo_pass|fail]:def:1
def oval_eval_sample(oval_defs):
    states = {'false': 0, 'true': 0, 'err': 0, 'unknown': 0, 'neval': 0,
              'na': 0, 'verbose': True, 'results': []}

    sess = oscap.oval.agent_new_session(oval_defs, "")
    ret = oscap.oval.agent_eval_system(sess, oval_sample_callback, states)

    for tmp in states['results']:
        if isinstance(tmp, Exception):
            raise tmp
        else:
            print(tmp)

'''
       ================        MAIN TEST           ====================
'''


print("Opening oval file with original C functions ... ")
oval_defs = oscap.oval.definition_model_import_source(
    oscap.common.source_new_from_file(get_path("samples/oval_sample.xml")))

oval_eval_sample(oval_defs)

# TODO: do the same thing with oval_import



#!/usr/bin/env python

# Author:
#   Dominique Blaze <contact@d0m.tech>
#
# Description:
#   Test introspection features of Python API
#    - Test the presence or the absence of expected builtins functions
#    - Indentify a constant name by its value and prefix (prefix XCCDF_RESULT
#       with value 1 (usually) should identify XCCDF_RESULT_PASS
#    - Check presence of constants filtered by a prefix (XCCDF_RESULT with
#       value = None should returns all XCCDF_RESULT_* constants).
#


from import_handler import oscap
from pprint import pprint

oval_funcs = oscap.oval.introspect_functions()

if oval_funcs.get('oval_variable_get_values') is None:
    raise Exception("method 'oval_variable_get_values' not found in builtins"
                    "functions of oval object (should be in oscap.oval.introspect_functions())")

if oval_funcs.get('xccdf_check_get_id') is not None:
    raise Exception("method 'xccdf_check_get_id' should not be found in "
                    "oscap.oval.introspect_functions(), because the prefix isn't 'oval')")

if oscap.oval.introspect_all().get('xccdf_check_get_id') is None:
    raise Exception("method 'xccdf_check_get_id' should be present in "
                    "oscap.oval.introspect_all(), which returns all available builtins functions")

print("Introspection of builtins functions seems working.")


# should returns {'XCCDF_RESULT_PASS': 1}
# but it's not impossible that the numeric value change (in fact we don't care of it)
pass_val = oscap.oval.XCCDF_RESULT_PASS  # is usually 1

var1 = oscap.oval.introspect_constants(pass_val, "XCCDF_RESULT")
if len(var1) == 1 and list(var1.keys())[0] == "XCCDF_RESULT_PASS":
    print("Introspection of oscap.oval.XCCDF_RESULT_PASS ok")


xccdf_results = oscap.oval.introspect_constants(None, "XCCDF_RESULT")
# should returns something like {
# 'XCCDF_RESULT_ERROR': 3,
# 'XCCDF_RESULT_FAIL': 2,
# 'XCCDF_RESULT_FIXED': 9,
# 'XCCDF_RESULT_INFORMATIONAL': 8,
# 'XCCDF_RESULT_NOT_APPLICABLE': 5,
# 'XCCDF_RESULT_NOT_CHECKED': 6,
# 'XCCDF_RESULT_NOT_SELECTED': 7,
# 'XCCDF_RESULT_PASS': 1,
# 'XCCDF_RESULT_UNKNOWN': 4}

expected_constants = ('XCCDF_RESULT_ERROR', 'XCCDF_RESULT_FAIL', 'XCCDF_RESULT_FIXED',
                      'XCCDF_RESULT_INFORMATIONAL', 'XCCDF_RESULT_NOT_APPLICABLE',
                      'XCCDF_RESULT_NOT_CHECKED', 'XCCDF_RESULT_PASS',
                      'XCCDF_RESULT_NOT_SELECTED', 'XCCDF_RESULT_UNKNOWN')

for c in expected_constants:
    if c not in xccdf_results:
        raise Exception("oscap.oval.introspect_constants(None, 'XCCDF_RESULT) should"
                        "contains the constant {0}.".format(c))

print("Introspection of constants ok with prefix XCCDF_RESULT_*")
